@@ -2,27 +2,34 @@
 #include "ui_travelsearchwindow.h"
 #include "searchdialog.h"
 #include "travelinfodialog.h"
+#include "recentwindow.h"
 
 #include <QtXml>
 
 TravelSearchWindow::TravelSearchWindow(QWidget *parent) :
-    QMainWindow(parent),
-    ui(new Ui::TravelSearchWindow)
+        QMainWindow(parent),
+        ui(new Ui::TravelSearchWindow)
 {
 #ifdef Q_WS_MAEMO_5
     setAttribute(Qt::WA_Maemo5StackedWindow);
     setAttribute(Qt::WA_Maemo5AutoOrientation, true);
 #endif
     ui->setupUi(this);
+    ui->lblNoResultsFounds->hide();
 
     timePicker = new QMaemo5TimePickSelector(this);
     datePicker = new QMaemo5DatePickSelector(this);
-    QMaemo5ValueButton *dateButton = new QMaemo5ValueButton("Date", this);
-    QMaemo5ValueButton *timeButton = new QMaemo5ValueButton("Time", this);
+    dateButton = new QMaemo5ValueButton(tr("Date"), this);
+    timeButton = new QMaemo5ValueButton(tr("Time"), this);
     dateButton->setPickSelector(datePicker);
     timeButton->setPickSelector(timePicker);
-    ui->dateTimeContainer->addWidget(dateButton);
-    ui->dateTimeContainer->addWidget(timeButton);
+    ui->dateTimeHorizontalContainer->removeWidget(ui->btnNow);
+    ui->dateTimeHorizontalContainer->addWidget(dateButton);
+    ui->dateTimeHorizontalContainer->addWidget(timeButton);
+    ui->dateTimeHorizontalContainer->addWidget(ui->btnNow);
+
+    searchToDialog = new SearchDialog(this);
+    searchFromDialog = new SearchDialog(this);
 
     manager = new QNetworkAccessManager(this);
     ui->tblResults->horizontalHeader()->setResizeMode(QHeaderView::Stretch);
@@ -37,17 +44,38 @@ void TravelSearchWindow::orientationChanged() {
     QRect screenGeometry = QApplication::desktop()->screenGeometry();
     if (screenGeometry.width() > screenGeometry.height()) {
         portraitMode = false;
+        ui->dateTimeVerticalContainer->removeWidget(dateButton);
+        ui->dateTimeHorizontalContainer->removeWidget(timeButton);
+        ui->dateTimeHorizontalContainer->removeWidget(ui->btnNow);
+        ui->dateTimeHorizontalContainer->addWidget(dateButton);
+        ui->dateTimeHorizontalContainer->addWidget(timeButton);
+        ui->dateTimeHorizontalContainer->addWidget(ui->btnNow);
     } else {
         portraitMode = true;
+        ui->dateTimeVerticalContainer->removeWidget(dateButton);
+        ui->dateTimeHorizontalContainer->removeWidget(ui->btnNow);
+        ui->dateTimeHorizontalContainer->removeWidget(timeButton);
+        ui->dateTimeVerticalContainer->insertWidget(0, dateButton);
+        ui->dateTimeHorizontalContainer->addWidget(timeButton);
+        ui->dateTimeHorizontalContainer->addWidget(ui->btnNow);
     }
 }
+
+void TravelSearchWindow::setPlaceFrom(Place place) {
+    this->placeFrom = place;
+    ui->btnPlaceFrom->setText(place.placeName);
+}
+
+void TravelSearchWindow::setPlaceTo(Place place) {
+    this->placeTo = place;
+    ui->btnPlaceTo->setText(place.placeName);
+}
+
 void TravelSearchWindow::setPlace(Place place, bool isFrom) {
     if(isFrom) {
-        this->placeFrom = place;
-        ui->btnPlaceFrom->setText(place.placeName);
+        setPlaceFrom(place);
     } else {
-        this->placeTo = place;
-        ui->btnPlaceTo->setText(place.placeName);
+        setPlaceTo(place);
     }
 }
 
@@ -68,8 +96,7 @@ void TravelSearchWindow::changeEvent(QEvent *e)
     }
 }
 
-Place TravelSearchWindow::searchPlace() {
-    SearchDialog *search = new SearchDialog(this);
+Place TravelSearchWindow::searchPlace(SearchDialog *search) {
     search->setNormalSearch();
     if(portraitMode) {
         search->setAttribute(Qt::WA_Maemo5PortraitOrientation, true);
@@ -87,7 +114,7 @@ Place TravelSearchWindow::searchPlace() {
 
 void TravelSearchWindow::on_btnPlaceFrom_clicked()
 {
-    Place place = searchPlace();
+    Place place = searchPlace(searchFromDialog);
     if(place.placeId != 0) {
         ui->btnPlaceFrom->setText(place.placeName);
         placeFrom = place;
@@ -96,7 +123,7 @@ void TravelSearchWindow::on_btnPlaceFrom_clicked()
 
 void TravelSearchWindow::on_btnPlaceTo_clicked()
 {
-    Place place = searchPlace();
+    Place place = searchPlace(searchToDialog);
     if(place.placeId != 0) {
         ui->btnPlaceTo->setText(place.placeName);
         placeTo = place;
@@ -105,6 +132,13 @@ void TravelSearchWindow::on_btnPlaceTo_clicked()
 
 void TravelSearchWindow::on_pushButton_clicked()
 {
+    ui->lblNoResultsFounds->hide();
+    Search *search = new Search();
+    search->placeFrom = placeFrom;
+    search->placeTo = placeTo;
+    search->type = Search::Travel;
+    Search::savePrepended(search);
+
     QString dataUrl = "http://reis.trafikanten.no/topp2009/topp2009ws.asmx"; //
     QNetworkRequest request = QNetworkRequest(QUrl(dataUrl));
     qDebug() << "requesting" << dataUrl;
@@ -120,8 +154,8 @@ void TravelSearchWindow::on_pushButton_clicked()
                    "</soap12:Envelope>";
     qDebug() << "request data" << data;
     request.setHeader(QNetworkRequest::ContentTypeHeader, "text/xml; charset=\"UTF-8\"");
-    request.setHeader(QNetworkRequest::ContentLengthHeader, data.toLatin1().length());
-    manager->post(request, data.toLatin1());
+    request.setHeader(QNetworkRequest::ContentLengthHeader, data.toUtf8().length());
+    manager->post(request, data.toUtf8());
     setAttribute(Qt::WA_Maemo5ShowProgressIndicator, true);
 }
 
@@ -139,6 +173,11 @@ void TravelSearchWindow::replyFinished(QNetworkReply *reply) {
         if(travel.isNull()) {
             qDebug() << "No travel proposals";
         }
+        if(travel.isNull() || response.isNull()) {
+            ui->lblNoResultsFounds->show();
+        } else {
+            ui->lblNoResultsFounds->hide();
+        }
         int row = 0;
         QList<Travel*> travels;
         while(!travel.isNull()) {
@@ -213,7 +252,7 @@ QVariant TravelListModel::data(const QModelIndex &index, int role) const {
 }
 
 void TravelListDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
-                              const QModelIndex &index) const {
+                               const QModelIndex &index) const {
     QStyledItemDelegate::paint(painter, option, index);
 
     Travel *e = qVariantValue<Travel *>(index.data());
@@ -225,8 +264,8 @@ void TravelListDelegate::paint(QPainter *painter, const QStyleOptionViewItem &op
     painter->save();
 
     painter->drawText(rect, Qt::AlignTop | Qt::AlignLeft, e->departureTime.toString("hh:mm") + " - " + e->arrivalTime.toString("hh:mm"));
-    if(e->travelStages.size() > 1) {
-        painter->drawText(rect, Qt::AlignTop | Qt::AlignRight, tr("%n switche(s)", "", e->travelStages.size() - 1));
+    if(e->travelStagesNum > 1) {
+        painter->drawText(rect, Qt::AlignTop | Qt::AlignRight, tr("%n switche(s)", "", e->travelStagesNum - 1));
     }
 
     painter->setPen(option.palette.mid().color());
@@ -251,3 +290,84 @@ void TravelSearchWindow::on_tblResults_clicked(QModelIndex index)
     }
     dialog->exec();
 }
+
+void TravelSearchWindow::on_actionAddFavorite_triggered()
+{
+    Search *search = new Search();
+    search->placeFrom = placeFrom;
+    search->placeTo = placeTo;
+    search->type = Search::Travel;
+    Search::savePrepended(search, Search::Favorites);
+}
+
+void TravelSearchWindow::on_actionSwitch_direction_triggered()
+{
+    Place tmpPlace = placeFrom;
+    placeFrom = placeTo;
+    placeTo = tmpPlace;
+    ui->btnPlaceFrom->setText(placeFrom.placeName);
+    ui->btnPlaceTo->setText(placeTo.placeName);
+}
+
+void TravelSearchWindow::favoritePlaceSelected(Place place) {
+    if(favoriteSelectMode == FavoriteFrom) {
+        placeFrom = place;
+        ui->btnPlaceFrom->setText(place.placeName);
+    } else if(favoriteSelectMode == FavoriteTo) {
+        placeTo = place;
+        ui->btnPlaceTo->setText(place.placeName);
+    }
+}
+
+void TravelSearchWindow::showFavoriteMessage(Place place, int mode) {
+
+    bool listFavorites = false;
+    Search *search = new Search();
+    search->placeFrom = place;
+    search->placeTo = Place();
+    search->type = Search::Realtime;
+    bool contains = Search::contains(Search::favoritesRealtime(), search);
+    if(!place.isNull() && !contains) {
+        QMessageBox msg(this);
+        msg.setText(tr("Do you wish to save the current selection as a favorite or select "
+                       "a different place from your list of favorites?"));
+        msg.setWindowTitle(tr("Save selection or find favorite?"));
+        QPushButton *saveButton = msg.addButton("Save as favorite", QMessageBox::YesRole);
+        QPushButton *selectButton = msg.addButton("Select favorite", QMessageBox::YesRole);
+        msg.setStandardButtons(QMessageBox::Cancel);
+        msg.exec();
+        if(msg.clickedButton() == saveButton) {
+            Search::savePrepended(search, Search::Favorites);
+        } else if(msg.clickedButton() == selectButton) {
+            listFavorites = true;
+        }
+    }
+
+    if(place.isNull() || contains || listFavorites){
+        favoriteSelectMode = mode;
+        RecentWindow* win = new RecentWindow(RecentWindow::FavoritesRealtime, this);
+        connect(win, SIGNAL(placeSelected(Place)), SLOT(favoritePlaceSelected(Place)));
+        if(portraitMode) {
+            win->setAttribute(Qt::WA_Maemo5PortraitOrientation, true);
+        } else {
+            win->setAttribute(Qt::WA_Maemo5LandscapeOrientation, true);
+        }
+        win->show();
+    }
+}
+
+void TravelSearchWindow::on_btnFromFavorite_clicked()
+{
+    showFavoriteMessage(placeFrom, FavoriteFrom);
+}
+
+void TravelSearchWindow::on_btnToFavorite_clicked()
+{
+    showFavoriteMessage(placeTo, FavoriteTo);
+}
+
+void TravelSearchWindow::on_btnNow_clicked()
+{
+    datePicker->setCurrentDate(QDate::currentDate());
+    timePicker->setCurrentTime(QTime::currentTime());
+}
